/*
 * multican 0.0.5, USB remote control tool for Canon cameras.
 *
 * Copyright (C) 2006 Jindrich Novy (jnovy@users.sourceforge.net)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef USBIO_H_INCLUDED
#define USBIO_H_INCLUDED 1

#define _GNU_SOURCE

#include <usb.h>

#define USB_TIMEOUT     15000   /* miliseconds */
#define USB_MAX_DEVICES 6

#define USB_READ        0xc0
#define USB_WRITE       0x40
#define USB_SINGLE_BYTE 0x0c
#define USB_MORE_BYTES  0x04

#define USB_READ_BYTE(handle,value,byteptr) \
        usb_control_msg(handle, USB_READ, USB_SINGLE_BYTE, value, 0, (char*)byteptr, 1, USB_TIMEOUT)
#define USB_READ_BYTES(handle,value,bytes,size) \
        usb_control_msg(handle, USB_READ, size>1?USB_MORE_BYTES:USB_SINGLE_BYTE, value, 0, (char*)bytes, size, USB_TIMEOUT)
#define USB_WRITE_BYTE(handle,value,byteptr) \
        usb_control_msg(handle, USB_WRITE, USB_SINGLE_BYTE, value, 0, (char*)byteptr, 1, USB_TIMEOUT)
#define USB_WRITE_BYTES(handle,value,bytes,size) \
        usb_control_msg(handle, USB_WRITE, size>1?USB_MORE_BYTES:USB_SINGLE_BYTE, value, 0, (char*)bytes, size, USB_TIMEOUT)

#define CANON_CLASS(c) (camera_id[(c)->cam_id].Class)
#define IS_EOS(c) (CANON_CLASS(c) == 4 || CANON_CLASS(c) == 6)
#define IS_TYPE(c,s) (strstr(camera_id[(c)->cam_id].name, (s)) != NULL)

typedef struct {
        usb_dev_handle *h;
        struct usb_device *dev;
        struct usb_interface *interface;
        struct usb_interface_descriptor *desc;
        int bulk_in;
        int bulk_out;
        int bulk_int_in;
        int cam_id;
        char canon_state;
        char *camera_type;
        char *owner;
        char *firmware;
        int  firmware1;
        int  firmware2;
        int  firmware3;
        int  firmware4;
        int identified;
        char *power_status;
        char *power_type;
        unsigned body_id;
        int got_body_id;
        int flash_state;
        int thumb_to_pc;
        int full_to_pc;
        int thumb_to_drive;
        int full_to_drive;
        unsigned avail_shots;
        int ready;
        unsigned max_transfer;
        unsigned char rel_params[0x30];
        int rel_params_filled;
        int rel_params_updated;
} camera;

static struct {
        unsigned idVendor, idProduct, Class;
        char *name;
} camera_id[] = {
        {0x04a9, 0x3041, 0, "PowerShot S10"},
        {0x04a9, 0x3043, 0, "PowerShot S20"},
        {0x04a9, 0x3044, 4, "EOS D30"},
        {0x04a9, 0x3045, 0, "PowerShot S100"},
        {0x04a9, 0x3046, 0, "IXY Digital"},
        {0x04a9, 0x3047, 0, "Digital IXUS"},
        {0x04a9, 0x3048, 0, "PowerShot G1"},
        {0x04a9, 0x3049, 0, "PowerShot Pro90 IS"},
        {0x04a9, 0x304b, 1, "IXY Digital 300"},
        {0x04a9, 0x304c, 1, "PowerShot S300"},
        {0x04a9, 0x304d, 1, "Digital IXUS 300"},
        {0x04a9, 0x304e, 1, "PowerShot A20"},
        {0x04a9, 0x304f, 1, "PowerShot A10"},
        {0x04a9, 0x3050, 1, "PowerShot unknown 1"},
        {0x04a9, 0x3051, 0, "PowerShot S110"},
        {0x04a9, 0x3052, 0, "Digital IXUS V"},
        {0x04a9, 0x3055, 1, "PowerShot G2"},
        {0x04a9, 0x3056, 1, "PowerShot S40"},
        {0x04a9, 0x3057, 1, "PowerShot S30"},
        {0x04a9, 0x3058, 1, "PowerShot A40"},
        {0x04a9, 0x3059, 1, "PowerShot A30"},
        {0x04a9, 0x305b, 1, "ZR45MC Digital Camcorder"},
        {0x04a9, 0x3060, 4, "EOS D60"},
        {0x04a9, 0x3061, 1, "PowerShot A100"},
        {0x04a9, 0x3062, 1, "PowerShot A200"},
        {0x04a9, 0x3065, 1, "PowerShot S200 or Digital IXUS V2"},
        {0x04a9, 0x3066, 1, "PowerShot S330 or Digital IXUS 330"},
        {0x04a9, 0x3067, 1, "MV550i Digital Video Camera"},
        {0x04a9, 0x3069, 1, "PowerShot G3"},
        {0x04a9, 0x306a, 1, "Digital unknown 3"},
        {0x04a9, 0x306b, 1, "Optura 200 MC or IXY2i or MVX2i Digital Video Camera"},
        {0x04a9, 0x306c, 5, "PowerShot S45"},
        {0x04a9, 0x306d, 5, "PowerShot S45 (ptp)"},
        {0x04a9, 0x306e, 5, "PowerShot G3"},
        {0x04a9, 0x306f, 5, "PowerShot G3 (ptp)"},
        {0x04a9, 0x3070, 4, "PowerShot S230 or Digital IXUS V3"},
        {0x04a9, 0x3071, 4, "PowerShot S230 (ptp) or Digital IXUS V3 (ptp)"},
        {0x04a9, 0x3072, 5, "PowerShot SD100 or Digital IXUS II"},
        {0x04a9, 0x3073, 1, "PowerShot A70"},
        {0x04a9, 0x3074, 1, "PowerShot A60"},
        {0x04a9, 0x3075, 1, "Digital IXUS 400 Camera or PowerShot S400"},
        {0x04a9, 0x3076, 1, "PowerShot A300"},
        {0x04a9, 0x3077, 4, "PowerShot S50"},
        {0x04a9, 0x3078, 1, "ZR70MC Digital Camcorder"},
        {0x04a9, 0x3079, 1, "MV650i Digital Video Camera (ptp)"},
        {0x04a9, 0x307a, 1, "MV650i Digital Video Camera"},
        {0x04a9, 0x307b, 1, "MV630i Digital Video Camera (ptp)"},
        {0x04a9, 0x307c, 1, "MV630i Digital Video Camera"},
        {0x04a9, 0x307f, 1, "Optura 20 (ptp) or MVX150i (ptp)"},
        {0x04a9, 0x3080, 1, "Optura 20 or MVX150i"},
        {0x04a9, 0x3081, 1, "Optura 10 or MVX100i"},
        {0x04a9, 0x3082, 1, "Optura 10 or MVX100i"},
        {0x04a9, 0x3083, 4, "EOS 10D"},
        {0x04a9, 0x3084, 4, "EOS 300D or Digital Rebel or Kiss Digital"},
        {0x04a9, 0x3085, 5, "PowerShot G5"},
        {0x04a9, 0x3087, 1, "Elura 50 (ptp)"},
        {0x04a9, 0x3088, 1, "Elura 50"},
        {0x04a9, 0x308d, 1, "Optura Xi (ptp) or MVX 3i (ptp) or FV M1 (ptp)"},
        {0x04a9, 0x308e, 1, "Optura Xi or MVX 3i or FV M1"},
        {0x04a9, 0x3095, 1, "Optura 300 (ptp) or MVX 10i (ptp) or IXY DV M2 (ptp)"},
        {0x04a9, 0x3096, 1, "Optura 300 or MVX 10i or IXY DV M2"},
        {0x04a9, 0x3099, 4, "EOS 300D (ptp)"},
        {0x04a9, 0x309a, 1, "PowerShot A80"},
        {0x04a9, 0x309b, 1, "SD10 Digital ELPH or Digital IXUS i or IXY Digital L"},
        {0x04a9, 0x309c, 5, "PowerShot S1 IS"},
        {0x04a9, 0x30a9, 1, "Optura 40/MVX25i camcorder"},
        {0x04a9, 0x30b1, 5, "PowerShot S70"},
        {0x04a9, 0x30b2, 5, "PowerShot S60"},
        {0x04a9, 0x30b3, 5, "PowerShot G6"},
        {0x04a9, 0x30b4, 5, "Digital IXUS 500 or PowerShot S500 Digital ELPH"},
        {0x04a9, 0x30b5, 1, "PowerShot A75"},
        {0x04a9, 0x30b6, 1, "PowerShot SD110 Digital ELPH or Digital IXUS IIs"},
        {0x04a9, 0x30b7, 5, "PowerShot A400"},
        {0x04a9, 0x30b8, 5, "PowerShot A310"},
        {0x04a9, 0x30b9, 5, "Powershot A85"},
        {0x04a9, 0x30ba, 5, "PowerShot S410 Digital ELPH or Digital IXUS 430 or IXY Digital 430"},
        {0x04a9, 0x30bb, 5, "PowerShot A95"},
        {0x04a9, 0x30bc, 4, "EOS 10D (ptp)"},
        {0x04a9, 0x30bf, 6, "PowerShot SD300 or Digital IXUS 40"}, /* IXUS likely in ptp */
        {0x04a9, 0x30c0, 6, "PowerShot SD200 or Digital IXUS 30 or IXY Digital 40"},
        {0x04a9, 0x30c1, 4, "PowerShot SD400 or Digital IXUS 50 or IXY Digital 55"},
        {0x04a9, 0x30c2, 1, "PowerShot A510"},
        {0x04a9, 0x30c4, 5, "PowerShot SD20 or Digital IXUS i5 or IXY Digital L2"},
        {0x04a9, 0x30e9, 6, "EOS 1D Mark II"},
        {0x04a9, 0x30ea, 6, "EOS 1D Mark II (ptp)"},
        {0x04a9, 0x30eb, 6, "EOS 20D"},
        {0x04a9, 0x30ec, 6, "EOS 20D (ptp)"},
        {0x04a9, 0x30ee, 6, "EOS 350D or Digital Rebel XT or EOS Kiss Digital N"},
        {0x04a9, 0x30ef, 6, "EOS 350D (ptp) or Digital Rebel XT (ptp) or EOS Kiss Digital N (ptp)"},
        {0x04a9, 0x30f0, 1, "PowerShot S2 IS"},
        {0x04a9, 0x30f2, 5, "PowerShot SD500 or Digital IXUS 700 or IXY Digital 600"},
        {0x04a9, 0x30fd, 5, "PowerShot A610"},
        {0x04a9, 0x3101, 6, "EOS 5D"},
        {0x04a9, 0x3110, 6, "EOS 400D"},
        {0x04a9, 0x3113, 6, "EOS 30D"},
        {0x04a9, 0x3114, 6, "EOS 30D (ptp)"},
        {0x04a9, 0x311a, 4, "PowerShot S3 IS"},
        {0x04a9, 0x3145, 6, "EOS 450D"},
        {0x04a9, 0x3199, 6, "EOS 5D Mark II"},
        {0, 0, 0, NULL}
};

static struct {
        unsigned code;
        char *text;
} canon_status[] = {
        {0x02000022, "File not found"},
        {0x02000029, "File was protected"},
        {0x0200002a, "Compact Flash card full"},
        {0x02000081, "Failed to lock EOS keys"},
        {0x02000082, "Failed to unlock EOS keys"},
        {0x02000085, "Could not switch to capture mode"},
        {0x02000086, "Invalid command parameters"},
        {0x00000086, "Can't unlock EOS keys (new)"},
        {0x02000087, "No storage card in camera"},
        {0x82200040, "Unknown error (new protocol)"},
        {0x82220040, "Unknown error (new protocol)"},
        {0x00000000, NULL}
};

extern unsigned usb_sequence;
extern int tolerate_error, retries;

void	 usb_hexdump( char *bytes, int size );
unsigned get_le32( char *b );
void	 put_le32( char *b, unsigned v );

/* Canon low-level USB comminication routines */
void usb_read_byte( usb_dev_handle *h, int value, char *byte );
void usb_read_bytes( usb_dev_handle *h, int value, char *bytes, int size );
void usb_write_byte( usb_dev_handle *h, int value, char *byte );
void usb_write_bytes( usb_dev_handle *h, int value, char *bytes, int size );
void usb_bulk_read_bytes( usb_dev_handle *h, int ep, char *bytes, int size );
void usb_send_packet( camera *c, unsigned char cmd1, unsigned char cmd2, unsigned cmd3, char *arguments, int argsize );
void usb_receive_packet( camera *c, char *bytes, int size );

#endif
